/*
 * $Id: igmp.h,v 1.1 1999/05/22 10:04:44 masaki Exp $
 */

#ifndef _IGMP_H
#define _IGMP_H

#include <config.h>
#include <linked_list.h>
#include <trace.h>
#include <timer.h>
#include <schedule.h>
#include <hash.h>

#include <mrt.h>
#include <proto.h>
#include <interface.h>
#include <rib.h>

#ifdef HAVE_IFF_MULTICAST
#define IGMP_MAX_PDU		4096 /* v4 igmp and v6 icmp group */

#define	IGMP_ROBUSTNESS_VARIABLE		2
#define	IGMP_QUERY_INTERVAL			125
#define	IGMP_QUERY_RESPONSE_INTERVAL		10
#define	IGMP_GROUP_MEMBERSHIP_INTERVAL		\
	    (IGMP_ROBUSTNESS_VARIABLE * IGMP_QUERY_INTERVAL + \
					IGMP_QUERY_RESPONSE_INTERVAL)
#define	IGMP_OTHER_QUERIER_PRESENT_INTERVAL	\
	    (IGMP_ROBUSTNESS_VARIABLE * IGMP_QUERY_INTERVAL + \
					IGMP_QUERY_RESPONSE_INTERVAL / 2)
#define	IGMP_STARTUP_QUERY_INTERVAL	(IGMP_QUERY_INTERVAL / 4)
#define	IGMP_STARTUP_QUERY_COUNT		IGMP_ROBUSTNESS_VARIABLE
#define	IGMP_LAST_MEMBER_QUERY_INTERVAL		1
#define	IGMP_LAST_MEMBER_QUERY_COUNT		IGMP_ROBUSTNESS_VARIABLE


typedef struct _igmp_t {
    int proto;			/* PROTO_IGMP or PROTO_IGMPV6 */
    inteface_bitset_t interface_mask;	/* mask of interfaces pim is configed for */
    LINKED_LIST *ll_interfaces;	/* enabled pim interface */
    HASH_TABLE *hash;		/* hash of prefixes */
    trace_t *trace;

    schedule_t *schedule;
    mtimer_t *timer;		/* timer used for sending update */
    mtimer_t *age;		/* aging routes */
    mtimer_t *flash;		/* flash update */

    int sockfd;
    prefix_t *all_hosts;
    prefix_t *all_routers;
} igmp_t;


typedef struct _igmp_group_t {  
    prefix_t *group;
    prefix_t *reporter;
    time_t ctime;	/* time received */
    int interval;	/* timeout interval since group specific query on
			   non-querier router sets different timeout value */
} igmp_group_t; 


extern igmp_t *IGMP;
#ifdef HAVE_IPV6
extern igmp_t *IGMPv6;
#endif /* HAVE_IPV6 */

int igmp_send (igmp_t *mrtigmp, prefix_t *dst, prefix_t *group, 
	       int type, int code, char *data, int datalen, 
	       interface_t *interface);

int igmp_init (int proto, trace_t * tr);
void igmp_interface (int proto, interface_t *interface, int on);
int igmp_show_group (uii_connection_t *uii);
#ifdef HAVE_IPV6
int igmp6_show_group (uii_connection_t *uii);
#endif /* HAVE_IPV6 */
#endif /* HAVE_IFF_MULTICAST */

#endif /* _IGMP_H */
